﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8.8
@EndOfDesignText@
'B4i Client Library for B4XIDEViewer
'Version 1.20 - March 12, 2019
'Author: Mikael Osterhed (moster67) 

#Event: Connected() 
#Event: Disconnected() 

Sub Class_Globals
	Private ser As B4XSerializator
	Type DataToPass (caller As Object, port As Int, host As String)
	Private ImageInfo As Map
	Private ImageCounter As Int = 0
	Private ViewNetWork As ViewerNetwork
	Private TheCallingModule As Object
	Private TheEventName As String
End Sub


'Initializes the object. Set IP Address and Port Number of
'the computer where B4XIDEViewer is running. Default
'port is 51070 but may be changed in the B4XIDEViewer app.
'You should now monitor the connection-events by creating
'two separate subs named EventName_Connected and EventName_Disconnected. 
'In future versions of B4X products, this last step may be 
'available in the AutoComplete list. 
Public Sub Initialize(CallingModule As Object, EventName As String, IPAddress As String, PortToUse As Int)
	Dim TheData As DataToPass
	TheData.Initialize
	TheData.caller = Me
	TheData.port = PortToUse
	TheData.host = IPAddress
	ViewNetWork.Initialize(TheData)
	TheCallingModule = CallingModule
	TheEventName = EventName
End Sub

#Region network related code

Private Sub ShowSetState
	If ViewNetWork.connected Then
		'Log("We're connected")
		If SubExists(TheCallingModule, TheEventName & "_Connected", 0) Then
			CallSub(TheCallingModule, TheEventName & "_Connected")
		End If
	Else
		'Log("We're disconnected")
		If SubExists(TheCallingModule, TheEventName & "_Disconnected", 0) Then
			CallSub(TheCallingModule, TheEventName & "_Disconnected")
		End If
	End If
	Log("IP Address: " & ViewNetWork.server.GetMyIP)
End Sub

Public Sub Connect
	If ViewNetWork.connected = False Then
		CallSub(ViewNetWork, "ConnectToServer")
	Else
		CallSub(ViewNetWork, "Disconnect")
	End If
End Sub

Private Sub CheckConnection
	If ViewNetWork.connected = False Then
		CallSub(ViewNetWork, "ConnectToServer")
	End If
End Sub

Private Sub NewData (data() As Byte) 'incoming data - not in use
	Log("NewData() triggered")
End Sub


#End Region


'Passes a single B4XBitmap to be shown in B4XIDEViewer.
'Each Bitmap is sent separately over the network.
'GIF-format is not supported. Use the method LogGifImage instead.
Public Sub LogImg(BP As B4XBitmap)
	CheckConnection
	'create a map
	ImageInfo.Initialize
	ImageCounter = ImageCounter + 1
	ImageInfo.Put("Image" & ImageCounter, ImageToBytesPerFormat(BP,"png"))
	CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes(ImageInfo))
End Sub

'Passes a list of B4XBitmap(s) to be shown in B4XIDEViewer.
'The list is sent over the network and then unpacked.
'GIF-format is not supported. Use the method LogGifImage instead. 
Public Sub LogImgList(BitmapList As List)
	CheckConnection
	'create a new map reading from input map
	Dim DataList As List
	DataList.Initialize
	ImageInfo.Initialize
	For x = 0 To BitmapList.Size -1
		ImageCounter = ImageCounter + 1
		Dim ImageName As String = "Image" & ImageCounter
		ImageInfo.Put(ImageName, ImageToBytesPerFormat(BitmapList.Get(x),"png"))
	Next
	DataList.Add(ImageInfo)
	CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes(DataList))
End Sub


'Passes a saved image to be shown in B4XIDEViewer.
'Each image is sent separately over the network.
'The FileName is case-sensitive. Make sure to include also the suffix.
'In B4J, if you pass DirAssets as Directory, no check if file exists will take place and
'therefore you should use Try and Catch in your code to handle any exceptions.
'GIF-format is not supported. Use the method LogGifImage instead.
public Sub LogSavedImage(Dir As String, FileName As String)
	CheckConnection
	Dim tmpFileName As String = FileName.ToLowerCase
	Dim imageType As String
	If tmpFileName.EndsWith(".jpeg") Or tmpFileName.EndsWith(".jpg") Then
		imageType = "jpeg"
	else if tmpFileName.EndsWith(".png") Then
		imageType = "png"
	End If
	
	Dim GetBitmap As B4XBitmap
	If File.Exists(Dir,FileName) Then
		GetBitmap = LoadBitmap(Dir, FileName)
	Else 'file does not exist
		Log("File does not exist. Check Directory and spelling of FileName.")
		Return
	End If

	ImageInfo.Initialize
	ImageCounter = ImageCounter + 1
	ImageInfo.Put("Image" & ImageCounter, ImageToBytesPerFormat(GetBitmap,imageType))
	CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes(ImageInfo))
End Sub

'Passes a saved GIF-image to be shown in B4XIDEViewer.
'Each image is sent separately over the network.
'The FileName is case-sensitive. Make sure to include also the suffix.
'In B4J, if you pass DirAssets as Directory, no check if file exists will take place and
'therefore you should use Try and Catch in your code to handle any exceptions.
Public Sub LogGifImage(Dir As String, FileName As String )
	If FileName.ToLowerCase.EndsWith(".gif") Then
		CheckConnection
		'create a map
		ImageInfo.Initialize
		ImageCounter = ImageCounter + 1
		ImageInfo.Put("GifImg" & ImageCounter, FileToBytes(Dir,FileName))
		CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes(ImageInfo))
	Else
		Log("Gif image must have gif-extension")
		Return
	End If
End Sub

'Passes a single B4XCanvas to be shown in B4XIDEViewer.
'The B4XCanvas will be converted into a B4XBitmap and sent separately over the network.
Public Sub LogCanvas(cvs As B4XCanvas)
	CheckConnection
	'create a map
	ImageInfo.Initialize
	ImageCounter = ImageCounter + 1
	Dim newBitmap As B4XBitmap = cvs.CreateBitmap
	ImageInfo.Put("Image" & ImageCounter, ImageToBytesPerFormat(newBitmap,"png"))
	CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes(ImageInfo))
End Sub

'Passes StringData to be shown in B4XIDEViewer.
'Each StringData is sent separately over the network.
Public Sub LogStringData(StringData As String)
	CheckConnection
	'create a map
	ImageInfo.Initialize
	ImageInfo.Put("string_", StringData)
	CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes(ImageInfo))
End Sub

'Passes a saved Video/Audio file to be played in B4XIDEViewer.
'Each Video/Audio file is sent separately over the network.
Public Sub LogVideoAudio(Dir As String, VideoName As String )
	CheckConnection
	'create a map
	ImageInfo.Initialize
	ImageInfo.Put("video_" & VideoName, FileToBytes(Dir,VideoName))
	CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes(ImageInfo))
End Sub

'Passes a Streaming-URL to be streamed in B4XIDEViewer.
'The URL-link is passed on as a String and will stream from
'B4XIDEViewer if protocol and codec are supported. 
Public Sub LogVideoAudioStream(StreamingURL As String)
	CheckConnection
	CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes(StreamingURL))
End Sub

'Passes a saved file to be shown in B4XIDEViewer.
'Each file is sent separately over the network. This is like a file-transfer.
Public Sub LogFile(Dir As String, FileName As String )
	CheckConnection
	'create a map
	ImageInfo.Initialize
	ImageInfo.Put("file_" & FileName, FileToBytes(Dir,FileName))
	CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes(ImageInfo))
End Sub

'Clears all images shown in B4XIDEViewer.
Public Sub ClearAllImages
	CheckConnection
	ImageCounter = 0
	CallSub2(ViewNetWork, "SendData", ser.ConvertObjectToBytes("ClearAllImages"))
End Sub

Private Sub ImageToBytesPerFormat(Image As B4XBitmap, Format As String) As Byte()
	Dim out As OutputStream
	out.InitializeToBytesArray(0)
	If Format = "jpeg" Then
		Image.WriteToStream(out, 100, "JPEG")
	else if Format = "png" Then
		Image.WriteToStream(out, 100, "PNG")
	End If
	out.Close
	Return out.ToBytesArray
End Sub

Private Sub FileToBytes (Dir As String, FileName As String) As Byte()
	Return Bit.InputStreamToBytes(File.OpenInput(Dir, FileName))
End Sub


